/*
 * 05/28/2003
 *
 * vector.h
 * Classe permettant la gestion des vecteurs 3D 
 *
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef _VECTOR_H
#define _VECTOR_H

#include <math.h>
//#include "global.h"

// Type scalaire
typedef float scalar_t;

// ###############
// Classe CVecteur
// ###############
class CVector
{
public:
	
	// Les 3 composantes d'un vecteur
	scalar_t x, y, z;

	// =======================
	// Constructeur par defaut
	// =======================
	CVector() { }

	// ===========
	// Destructeur
	// ===========
	~CVector() { }

	// ====================================================================
	// Constructeur permetant de creer un vecteur avec des valeurs definies
	// ====================================================================
	CVector(scalar_t X, scalar_t Y, scalar_t Z) 
	{ 
		x = X; y = Y; z = Z;
	}

	// ===========================================================
	// Surdefinition de l'operateur +, pour l'addition de vecteurs
	// ===========================================================
	CVector operator+(CVector vVector)
	{
		return CVector(vVector.x + x, vVector.y + y, vVector.z + z);
	}

	// ================================================================
	// Surdefinition de l'operateur -, pour la soustraction de vecteurs
	// ================================================================
	CVector operator-(CVector vVector)
	{
		return CVector(x - vVector.x, y - vVector.y, z - vVector.z);
	}

	CVector operator-()
	{
		return CVector(-x, -y, -z);
	}
	
	// ===================================================================================
	// Surdefinition de l'operateur *, pour la multiplication d'un vecteur par un scalaire
	// ===================================================================================
	CVector operator*(scalar_t num)
	{
		return CVector(x * num, y * num, z * num);
	}

	// =============================================================================
	// Surdefinition de l'operateur /, pour la division d'un vecteur par un scalaire
	// =============================================================================
	CVector operator/(scalar_t num)
	{
		num = 1/num;
		return CVector(x * num, y * num, z * num);
	}

	// ================================================================================
	// Surdefinition de l'operateur ^, pour le produit vectoriel de vecteurs
	// Le produit Vectoriel renvoi le vecteur perpendiculaire aux 2 vecteurs multiplies
	// ================================================================================
	CVector operator^(CVector vVector)
	{
		CVector vNormal;
		// Valeur X:  (V1.y * V2.z) - (V1.z * V2.y)	
		vNormal.x = ((y * vVector.z) - (z * vVector.y));								
		// Valeur Y:  (V1.z * V2.x) - (V1.x * V2.z)
		vNormal.y = ((z * vVector.x) - (x * vVector.z));
		// Valeur Z:  (V1.x * V2.y) - (V1.y * V2.x)
		vNormal.z = ((x * vVector.y) - (y * vVector.x));
		return vNormal;		
	}

	// =================================================================
	// Surdefinition de l'operateur %, pour le "dot" produit de vecteurs
	// Le "dot" produit renvoi le cos de l'angle entre les 2 vecteurs
	// Attention les 2 vecteurs doivent etre de longueur 1 (normalise)
	// =================================================================
	const scalar_t operator%(const CVector& vVector) const
	{
		return x*vVector.x + y*vVector.y + z*vVector.z;
	}
	
	// =====================
	// Longueur d'un vecteur
	// =====================
	static scalar_t Length(CVector vNormal)
	{
		// length = sqrt(V.x^2 + V.y^2 + V.z^2)  V est le vecteur
		return (scalar_t)sqrt((vNormal.x * vNormal.x) + (vNormal.y * vNormal.y) + (vNormal.z * vNormal.z));
	}

	// ==========================
	// Normalisation d'un vecteur
	// ==========================
	static CVector Normalize(CVector vVector)
	{
		return vVector / Length(vVector);		
	}

};

#endif

